@@ -6,7 +6,7 @@ import warnings
 
 import cf_xarray as cfxr
 import numpy as np
-import scipy.sparse as sps
+import sparse as sps
 import xarray as xr
 from xarray import DataArray, Dataset
 
@@ -455,7 +455,7 @@ class BaseRegridder(object):
             raise TypeError('input must be numpy array, dask array, xarray DataArray or Dataset!')
 
     @staticmethod
-    def _regrid_array(indata, *, weights, shape_in, shape_out, sequence_in, skipna, na_thres):
+    def _regrid_array(indata, weights, *, shape_in, shape_out, sequence_in, skipna, na_thres):
 
         if sequence_in:
             indata = np.expand_dims(indata, axis=-2)
@@ -481,7 +481,6 @@ class BaseRegridder(object):
     @property
     def _regrid_kwargs(self):
         return {
-            'weights': self.weights,
             'sequence_in': self.sequence_in,
             'shape_in': self.shape_in,
             'shape_out': self.shape_out,
@@ -490,7 +489,7 @@ class BaseRegridder(object):
     def regrid_numpy(self, indata, skipna=False, na_thres=1.0):
         """See __call__()."""
         outdata = self._regrid_array(
-            indata, skipna=skipna, na_thres=na_thres, **self._regrid_kwargs
+            indata, self.weights.data, skipna=skipna, na_thres=na_thres, **self._regrid_kwargs
         )
         return outdata
 
@@ -504,7 +503,8 @@ class BaseRegridder(object):
         outdata = da.map_blocks(
             self._regrid_array,
             indata,
-            dtype=float,
+            self.weights.data,
+            dtype=indata.dtype,
             chunks=output_chunk_shape,
             skipna=skipna,
             na_thres=na_thres,
@@ -522,14 +522,17 @@ class BaseRegridder(object):
         dr_out = xr.apply_ufunc(
             self._regrid_array,
             dr_in,
+            self.weights,
             kwargs=kwargs,
-            input_core_dims=[input_horiz_dims],
+            input_core_dims=[input_horiz_dims, ('out_dim', 'in_dim')],
             output_core_dims=[temp_horiz_dims],
             dask='parallelized',
-            output_dtypes=[float],
-            output_sizes={
-                temp_horiz_dims[0]: self.shape_out[0],
-                temp_horiz_dims[1]: self.shape_out[1],
+            output_dtypes=[dr_in.dtype],
+            dask_gufunc_kwargs={
+                'output_sizes': {
+                    temp_horiz_dims[0]: self.shape_out[0],
+                    temp_horiz_dims[1]: self.shape_out[1],
+                },
             },
             keep_attrs=keep_attrs,
         )
@@ -552,17 +555,22 @@ class BaseRegridder(object):
         ]
         ds_in = ds_in.drop_vars(non_regriddable)
 
+        ds_dtypes = [d.dtype for d in ds_in.data_vars.values()]
+
         ds_out = xr.apply_ufunc(
             self._regrid_array,
             ds_in,
+            self.weights,
             kwargs=kwargs,
-            input_core_dims=[input_horiz_dims],
+            input_core_dims=[input_horiz_dims, ('out_dim', 'in_dim')],
             output_core_dims=[temp_horiz_dims],
             dask='parallelized',
-            output_dtypes=[float],
-            output_sizes={
-                temp_horiz_dims[0]: self.shape_out[0],
-                temp_horiz_dims[1]: self.shape_out[1],
+            output_dtypes=ds_dtypes,
+            dask_gufunc_kwargs={
+                'output_sizes': {
+                    temp_horiz_dims[0]: self.shape_out[0],
+                    temp_horiz_dims[1]: self.shape_out[1],
+                },
             },
             keep_attrs=keep_attrs,
         )
@@ -632,9 +640,11 @@ class BaseRegridder(object):
         """Save weights to disk as a netCDF file."""
         if filename is None:
             filename = self.filename
-        w = self.weights
+        w = self.weights.data
         dim = 'n_s'
-        ds = xr.Dataset({'S': (dim, w.data), 'col': (dim, w.col + 1), 'row': (dim, w.row + 1)})
+        ds = xr.Dataset(
+            {'S': (dim, w.data), 'col': (dim, w.coords[1, :] + 1), 'row': (dim, w.coords[0, :] + 1)}
+        )
         ds.to_netcdf(filename)
         return filename
 
@@ -980,14 +990,24 @@ class SpatialAverager(BaseRegridder):
                 ignore_degenerate=self.ignore_degenerate,
                 unmapped_to_nan=False,
             )
-            w_all = sps.hstack((reg_ext.weights.tocsc(), -reg_holes.weights.tocsc()))
+            w_all = xr.concat((reg_ext.weights, -reg_holes.weights), 'in_dim')
         else:
-            w_all = reg_ext.weights.tocsc()
+            w_all = reg_ext.weights
 
-        # Combine weights of same owner and normalize
+        # "Transpose" the data, weights generated before are mesh -> grid, we want grid -> mesh
+        w_all = w_all.rename(in_dim='out_dim', out_dim='in_dim')
+
+        # Combine weights of same owner
         weights = _combine_weight_multipoly(w_all, owners)
-        weights = weights.multiply(1 / weights.sum(axis=0))
-        return weights.tocoo().T
+        # Normalize weights
+        wsum = weights.sum('in_dim')
+        # All this only to change the fill_value...
+        wsum = wsum.copy(
+            data=sps.COO(wsum.data.coords, wsum.data.data, shape=wsum.data.shape, fill_value=1)
+        )
+        weights = weights / wsum
+        # Transpose to fit with the rest of xesmf.
+        return weights.transpose('out_dim', 'in_dim')
 
     def _get_default_filename(self):
         # e.g. bilinear_400x600_300x400.nc
