@@ -5,7 +5,6 @@ import cf_xarray as cfxr
 import dask
 import numpy as np
 import pytest
-import scipy.sparse as sps
 import xarray as xr
 from numpy.testing import assert_allclose, assert_almost_equal, assert_equal
 from shapely.geometry import MultiPolygon, Polygon
@@ -32,6 +31,7 @@ ds_out['data_ref'] = xe.data.wave_smooth(ds_out['lon'], ds_out['lat'])
 ds_in.coords['time'] = np.arange(7) + 1
 ds_in.coords['lev'] = np.arange(11) + 1
 ds_in['data4D'] = ds_in['time'] * ds_in['lev'] * ds_in['data']
+ds_in['data4D_f4'] = ds_in['data4D'].astype('f4')
 ds_out['data4D_ref'] = ds_in['time'] * ds_in['lev'] * ds_out['data_ref']
 
 # use non-divisible chunk size to catch edge cases
@@ -50,7 +50,7 @@ ds_savg = xr.Dataset(
         'lat_b': (('lat_b',), [0, 1, 2]),
         'lon_b': (('lon_b',), [0, 1, 2, 3]),
     },
-    data_vars={'abc': (('lon', 'lat'), [[1, 2], [3, 4], [2, 4]])},
+    data_vars={'abc': (('lon', 'lat'), [[1.0, 2.0], [3.0, 4.0], [2.0, 4.0]])},
 )
 polys = [
     Polygon([[0.5, 0.5], [0.5, 1.5], [1.5, 0.5]]),  # Simple triangle polygon
@@ -220,15 +220,16 @@ def test_to_netcdf_nans(tmp_path):
     e = xr.open_dataset(efn)
 
     # Reformat to sparse COO matrix
-    smat = sps.coo_matrix(
-        (e.S.values, (e.row.values - 1, e.col.values - 1)),
-        shape=[np.prod(ds_out['lon'].shape), np.prod(ds_in['lon'].shape)],
-    )
+    smat = xe.smm.read_weights(e, np.prod(ds_in['lon'].shape), np.prod(ds_out['lon'].shape))
     # Add NaNs to weights
     smat = xe.smm.add_nans_to_weights(smat)
     # Updating the dataset
     e_nans = xr.Dataset(
-        {'S': (['n_s'], smat.data), 'row': (['n_s'], smat.row + 1), 'col': (['n_s'], smat.col + 1)}
+        {
+            'S': (['n_s'], smat.data.data),
+            'row': (['n_s'], smat.data.coords[0, :] + 1),
+            'col': (['n_s'], smat.data.coords[1, :] + 1),
+        }
     )
     # Comparison
     xr.testing.assert_identical(x, e_nans)
@@ -276,6 +277,29 @@ def ds_2d_to_1d(ds):
     return ds_1d
 
 
+@pytest.mark.parametrize('dtype', ['float32', 'float64'])
+@pytest.mark.parametrize(
+    'data_in',
+    [
+        pytest.param(np.array(ds_in['data']), id='np.ndarray'),
+        pytest.param(xr.DataArray(ds_in['data']), id='xr.DataArray input'),
+        pytest.param(xr.Dataset(ds_in[['data']]), id='xr.Dataset input'),
+        pytest.param(ds_in['data'].chunk(), id='da.Array input'),
+    ],
+)
+def test_regridded_respects_input_dtype(dtype, data_in):
+    """Tests regridded output has same dtype as input"""
+    data_in = data_in.astype(dtype)
+    regridder = xe.Regridder(ds_in, ds_out, 'bilinear')  # Make this a fixture?
+    out = regridder(data_in)
+
+    if 'data' in data_in:
+        # When data_in is xr.Dataset, a mapping...
+        assert out['data'].dtype == data_in['data'].dtype
+    else:
+        assert out.dtype == data_in.dtype
+
+
 def test_regrid_with_1d_grid():
     ds_in_1d = ds_2d_to_1d(ds_in)
     ds_out_1d = ds_2d_to_1d(ds_out)
@@ -528,6 +552,9 @@ def test_regrid_dataset():
         decimal=10,
     )
 
+    assert ds_result['data4D'].dtype == np.dtype('f8')
+    assert ds_result['data4D_f4'].dtype == np.dtype('f4')
+
     # check metadata
     xr.testing.assert_identical(ds_result['time'], ds_in['time'])
     xr.testing.assert_identical(ds_result['lev'], ds_in['lev'])
